==> lm78-1.2.0pre5.tgz: lm78-1.2.0pre5/lm75.c <==
/* -*- linux-c -*- */


/*
    lm75.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1998  Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *  LM75 sensor driver for LINUX
 *
 * Copyright (c) 1998 Philip Edelbrock
 * 
 * part of a module which provides a 
 * /proc/sensors file which shows
 * some information about your system.
 * Voltages, temperature and fan speeds
 * are currently reported.
 *
 *
 * please report any bugs/patches to me:
 *  phil@netroedge.com
 *
 *
 * Reference:
 *   National Semiconductor
 *   LM75 Digital Temperature Sensor and Thermal Watchdog with
 *   Two-Wire Interface, PDF, April 1997.
 *   http://www.national.com/ds/LM/LM75.pdf
 *
 *   Intel
 *   82371AB PCI-TO-ISA / IDE XCELERATOR (PIIX4), PDF, April 1997.
 *   ftp://ftp.intel.com
 *   
 *   System Management Bus Specification, Rev. 1.0, 1996.
 *   Contributors:
 *   Benchmarq Microelectronics Inc ., Duracell Inc.,
 *   Energizer Power Systems, Intel Corporation, Linear Technology Corporation,
 *   Maxim Integrated Products, Mitsubishi Electric Corporation,
 *   National Semiconductor Corporation, Toshiba Battery Co.,
 *   Varta Batterie AG, All rights reserved.
 *   http://www.sbs-forum.org/
 */

#include <linux/config.h>

/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include "main.h"
#include "lm75.h"
#include "smbus.h"


int LM75_Init () {
	int ret;

   if ((ret = SMBus_Init()) == 1)
        printk("lm_sensors: SMBus Initialization failed: LM75 readings will be disabled.");

	 return ret;
}

void LM75_Cleanup () {
   SMBus_Cleanup();
}

/* Input: a LM75 register value.
   Output: the temperature in half-degrees */
static int calc_temp(int value) {
   value=value >> 7;
   if (value > 0x0191)
      value= - ( 0x0200 - value );
   return value;
}

int LM75_Print_Values (char *buf) {
  int i,value,addr;
  int len=0;

  if (LM_Sensor_Data.valid) {

/* The below must be moved to lm78.c once we have SMBus support there */
#ifdef DEBUG
    value=LM_Sensor_Data.lm78_smbus_addr;
    if (value != -1) {
	len += sprintf(buf+len, "SMBus: LM78 probe: 0x%X\n",value);
     } else {
     	len += sprintf(buf+len, "SMBus: LM78 probe: failed!\n");
     }
#endif

    for (i=0; i < MAX_LM75 ; i++) 
      if ((addr = LM_Sensor_Data.lm75[i].address) != LM75_NOT_PRESENT) {
        value=calc_temp(LM_Sensor_Data.lm75[i].temp);
        len += sprintf(buf+len, "LM75 address 0x%X reports %d.%d degrees C\n",
               addr, value  >> 1, (value & 1) * 5);
#ifdef DEBUG
        len += sprintf(buf+len, "LM75 reg 1 (configuration register) at address 0x%X reports 0x%X\n",
               addr,LM_Sensor_Data.lm75[i].config);
        value=calc_temp(LM_Sensor_Data.lm75[i].temphyst);   
        len += sprintf(buf+len, "LM75 reg 2 (Thyst) at address 0x%X reports %d.%d degrees C\n",
               addr,value  >> 1, (value & 1) * 5);
        value=calc_temp(LM_Sensor_Data.lm75[i].tempos);   
        len += sprintf(buf+len, "LM75 reg 3 (Tos) at address 0x%X reports %d.%d degrees C\n",
               addr,value  >> 1, (value & 1) * 5);
#endif
      }
  }
  return len;
}

void LM75_Update_Values (void) {

  int i;
  int value,smbus_addr;

/* Probe for LM78 (and read config register). This should be moved to lm78.c
   once we have SMBus support there. */
#ifdef DEBUG
  /*if (! LM_Sensor_Data.valid)
    LM_Sensor_Data.lm78_smbus_addr=SMBus_Read(0x2D,0x040,QUICK_READ);
  */
#endif

/* Some very ugly code: if we have probed a LM75 location before, and 
   nothing was found, we do not probe it again. */
  for (i = 0 ; i < 8 ; i ++)
    if (! LM_Sensor_Data.valid || 
          (LM_Sensor_Data.lm75[i].address != LM75_NOT_PRESENT)) {
      smbus_addr=i+0x048; 
      /* Clear configuration reg */
      SMBus_Write(smbus_addr,0x000001,BYTE_DATA);
      value=SMBus_Read(smbus_addr,0,WORD_DATA);
      if (value != -1) {
        LM_Sensor_Data.lm75[i].address=smbus_addr;
        LM_Sensor_Data.lm75[i].temp=value;
#ifdef DEBUG
        LM_Sensor_Data.lm75[i].config=SMBus_Read(smbus_addr,1,WORD_DATA);
        LM_Sensor_Data.lm75[i].temphyst=SMBus_Read(smbus_addr,2,WORD_DATA);
        LM_Sensor_Data.lm75[i].tempos=SMBus_Read(smbus_addr,3,WORD_DATA);
#endif
      } else
        LM_Sensor_Data.lm75[i].address=LM75_NOT_PRESENT;
    }
}

==> lm78-1.2.1.tgz: lm78-1.2.1/lm75.c <==
/* -*- linux-c -*- */


/*
    lm75.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1998  Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *  LM75 sensor driver for LINUX
 *
 * Copyright (c) 1998 Philip Edelbrock
 * 
 * part of a module which provides a 
 * /proc/sensors file which shows
 * some information about your system.
 * Voltages, temperature and fan speeds
 * are currently reported.
 *
 *
 * please report any bugs/patches to me:
 *  phil@netroedge.com
 *
 *
 * Reference:
 *   National Semiconductor
 *   LM75 Digital Temperature Sensor and Thermal Watchdog with
 *   Two-Wire Interface, PDF, April 1997.
 *   http://www.national.com/ds/LM/LM75.pdf
 *
 *   Intel
 *   82371AB PCI-TO-ISA / IDE XCELERATOR (PIIX4), PDF, April 1997.
 *   ftp://ftp.intel.com
 *   
 *   System Management Bus Specification, Rev. 1.0, 1996.
 *   Contributors:
 *   Benchmarq Microelectronics Inc ., Duracell Inc.,
 *   Energizer Power Systems, Intel Corporation, Linear Technology Corporation,
 *   Maxim Integrated Products, Mitsubishi Electric Corporation,
 *   National Semiconductor Corporation, Toshiba Battery Co.,
 *   Varta Batterie AG, All rights reserved.
 *   http://www.sbs-forum.org/
 */

#include <linux/config.h>

/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include "main.h"
#include "lm75.h"
#include "smbus.h"
#include "lm78_smbus.h"


int Init_LM75 () {
	int ret,i;

   if ((ret = Init_SMBus()) != 0)
        printk("lm_sensors: SMBus Initialization failed: LM75 readings will be disabled.");
   else {
/* Set the limits on LM75's */
      for (i=0x48;i <= 0x4F; i++) {
           if (read_smbus(i,0,WORD_DATA) != -1) { /* Test for existance first */
	     write_smbus(i,1 | (00 << 8),WORD_DATA); /* Set Config reg to 0x00 */
	     write_smbus(i,2 | (Thyst_DEFAULT << 8),WORD_DATA); /* Set Thyst=Thyst_DEFAULT */
	     write_smbus(i,3 | (Tos_DEFAULT << 8),WORD_DATA); /* Set Tos=Tos_DEFAULT   */
	   }
      }
   }
   return ret;
}

void Cleanup_LM75 () {
   Cleanup_SMBus();
}

/* Input: a LM75 register value.
   Output: the temperature in half-degrees */
int calc_temp(int value) {
   value=value >> 7;
   if (value > 0x0191)
      value= - ( 0x0200 - value );
   return value;
}

int print_lm75_values (char *buf) {
  int i,value,value2,value3,addr;
  int len=0;

  if (sensor_data.valid) {
    for (i=0; i < MAX_LM75 ; i++) 
      if ((addr = sensor_data.lm75[i].address) != LM75_NOT_PRESENT) {
        value=calc_temp(sensor_data.lm75[i].temp);
        value2=calc_temp(sensor_data.lm75[i].tempos);
        value3=calc_temp(sensor_data.lm75[i].temphyst);
        len += sprintf(buf+len, "LM75 address 0x%X reports %d.%d degrees C (Tos=%d.%d, Thyst=%d.%d)\n",
               addr, value  >> 1, (value & 1) * 5,value2  >> 1, (value2 & 1) * 5,value3  >> 1, (value3 & 1) * 5);
#ifdef DEBUG
        len += sprintf(buf+len, "LM75 reg 1 (configuration register) at address 0x%X reports 0x%X\n",
               addr,sensor_data.lm75[i].config);
#endif
      }
  }
  return len;
}

void update_lm75_values () {

  int i;
  int value,smbus_addr;

/* Some very ugly code: if we have probed a LM75 location before, and 
   nothing was found, we do not probe it again. */
  for (i = 0 ; i < 8 ; i ++)
    if (! sensor_data.valid || 
          (sensor_data.lm75[i].address != LM75_NOT_PRESENT)) {
      smbus_addr=i+0x048; 
      /* Clear configuration reg */
      write_smbus(smbus_addr,0x000001,BYTE_DATA);
      value=read_smbus(smbus_addr,0,WORD_DATA);
      if (value != -1) {
        sensor_data.lm75[i].address=smbus_addr;
        sensor_data.lm75[i].temp=value;
#ifdef DEBUG
        sensor_data.lm75[i].config=read_smbus(smbus_addr,1,WORD_DATA);
#endif
        sensor_data.lm75[i].temphyst=read_smbus(smbus_addr,2,WORD_DATA);
        sensor_data.lm75[i].tempos=read_smbus(smbus_addr,3,WORD_DATA);
      } else
        sensor_data.lm75[i].address=LM75_NOT_PRESENT;
    }
}
